# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"get_subs": "00_core.ipynb",
         "filter_workflow": "00_core.ipynb",
         "get_bfiles_tuple": "00_core.ipynb",
         "get_sub_gradfiles": "00_core.ipynb",
         "BIDS_metadata": "00_core.ipynb",
         "PipetographyBaseInputSpec": "00_core.ipynb",
         "MRCatInputSpec": "00_core.ipynb",
         "MRCatOutputSpec": "00_core.ipynb",
         "MRCat": "00_core.ipynb",
         "GradCatInputSpec": "00_core.ipynb",
         "GradCatOutputSpec": "00_core.ipynb",
         "GradCat": "00_core.ipynb",
         "aff2rigidInputSpec": "00_core.ipynb",
         "aff2rigidOutputSpec": "00_core.ipynb",
         "fslaff2rigid": "00_core.ipynb",
         "ConvertInputSpec": "00_core.ipynb",
         "ConvertOutputSpec": "00_core.ipynb",
         "Convert": "00_core.ipynb",
         "GradCheckInputSpec": "00_core.ipynb",
         "GradCheckOutputSpec": "00_core.ipynb",
         "GradCheck": "00_core.ipynb",
         "dwidenoiseInputSpec": "00_core.ipynb",
         "dwidenoiseOutputSpec": "00_core.ipynb",
         "dwidenoise": "00_core.ipynb",
         "dwipreprocInputSpec": "00_core.ipynb",
         "dwipreprocOutputSpec": "00_core.ipynb",
         "dwipreproc": "00_core.ipynb",
         "BiasCorrectInputSpec": "00_core.ipynb",
         "BiasCorrectOutputSpec": "00_core.ipynb",
         "BiasCorrect": "00_core.ipynb",
         "MRInfoInputSpec": "00_core.ipynb",
         "MRInfoOutputSpec": "00_core.ipynb",
         "MRInfo": "00_core.ipynb",
         "CheckFiniteInputSpec": "00_core.ipynb",
         "CheckFiniteOutputSpec": "00_core.ipynb",
         "CheckFinite": "00_core.ipynb",
         "CompareIfInputSpec": "00_core.ipynb",
         "CompareIfOutputSpec": "00_core.ipynb",
         "CompareIf": "00_core.ipynb",
         "CheckNIZInputSpec": "00_core.ipynb",
         "CheckNIZOutputSpec": "00_core.ipynb",
         "CheckNIZ": "00_core.ipynb",
         "RicianNoiseInputSpec": "00_core.ipynb",
         "RicianNoiseOutputSpec": "00_core.ipynb",
         "RicianNoise": "00_core.ipynb",
         "MRThresholdInputSpec": "00_core.ipynb",
         "MRThresholdOutputSpec": "00_core.ipynb",
         "MRThreshold": "00_core.ipynb",
         "DWINormalizeInputSpec": "00_core.ipynb",
         "DWINormalizeOutputSpec": "00_core.ipynb",
         "DWINormalize": "00_core.ipynb",
         "TransConvertInputSpec": "00_core.ipynb",
         "TransConvertOutputSpec": "00_core.ipynb",
         "TransConvert": "00_core.ipynb",
         "MRTransformInputSpec": "00_core.ipynb",
         "MRTransformOutputSpec": "00_core.ipynb",
         "MRTransform": "00_core.ipynb",
         "WMBinarizeInputSpec": "00_core.ipynb",
         "WMBinarizeOutputSpec": "00_core.ipynb",
         "WMBinarize": "00_core.ipynb",
         "MRRegridInputSpec": "00_core.ipynb",
         "MRRegridOutputSpec": "00_core.ipynb",
         "MRRegrid": "00_core.ipynb",
         "mask2seedtuple": "00_core.ipynb",
         "gmwmiInputSpec": "00_core.ipynb",
         "gmwmiOutputSpec": "00_core.ipynb",
         "gmwmi": "00_core.ipynb",
         "tckSIFT2InputSpec": "00_core.ipynb",
         "tckSIFT2OutputSpec": "00_core.ipynb",
         "tckSIFT2": "00_core.ipynb",
         "MakeConnectomeInputSpec": "00_core.ipynb",
         "MakeConnectomeOutputSpec": "00_core.ipynb",
         "MakeConnectome": "00_core.ipynb",
         "pipeline": "01_pipeline.ipynb",
         "PreProcNodes": "02_nodes.ipynb",
         "ACPCNodes": "02_nodes.ipynb",
         "PostProcNodes": "02_nodes.ipynb",
         "connectome": "03_connectomes.ipynb",
         "Make5ttFSLInputSpec": "00_core.ipynb",
         "Make5ttFSLOutputSpec": "00_core.ipynb",
         "Make5ttFSL": "00_core.ipynb"}

modules = ["core.py",
           "pipeline.py",
           "nodes.py",
           "connectomes.py"]

doc_url = "https://axiezai.github.io/pipetography/"

git_url = "https://github.com/axiezai/pipetography/tree/master/"

def custom_doc_links(name): return None
